@@ -501,6 +501,7 @@ double hmm_max_or_sum(HMM *hmm, double **full_scores, double **emission_scores,
   }
 
   if (mode == VITERBI) {
+    int initialized = 0;
     for (k = 0; k < lst_size(hmm->predecessors[i]); k++) {
       int pred;
       double candidate;
@@ -509,9 +510,10 @@ double hmm_max_or_sum(HMM *hmm, double **full_scores, double **emission_scores,
       candidate = full_scores[pred][j-1] + 
         hmm_get_transition_score(hmm, pred, i);
 
-      if (candidate > retval) {
+      if (candidate > retval || initialized == 0) {
         retval = candidate;
         backptr[i][j] = pred;
+        initialized=1;
       }
     }
   }
@@ -718,7 +720,7 @@ void hmm_train_update_counts(Matrix *trans_counts, Vector *state_counts,
 /* for debugging */
 void hmm_dump_matrices(HMM *hmm, double **emission_scores, int seqlen,
                        double **full_scores, int **backptr) {
-  FILE *F = fopen("hmm.debug", "w+");
+  FILE *F = phast_fopen("hmm.debug", "w+");
   int i, j;
   char tmpstr[50];
 
@@ -771,7 +773,7 @@ void hmm_dump_matrices(HMM *hmm, double **emission_scores, int seqlen,
     }
   }
 
-  fclose(F);
+  phast_fclose(F);
 }
 
 /* creates and returns a trivial HMM, with a single state that
