@@ -22,3 +22,6 @@ mm <- build.mm(ms, 3);
 #   Sites with a negative score are not returned unless 
 #   we set threshold=-Inf as a parameter.
 score.ms(ms, pwm, mm)
+unlink("pwm.meme")
+unlink("input.fas")
+
